//+------------------------------------------------------------------+
//|                                                        LibFF.mqh |
//|                                                         renexxxx |
//|                                http://www.stevehopwoodforex.com/ |
//| This library was developed by renexxxx from the                  |
//|    http://www.stevehopwoodforex.com/ forum.                      |
//|                                                                  |
//| version 1.0   initial release (RZ)                               |
//| version 2.0   Rewrite using EasyXML                              |
//| version 2.1   Removed a little bug                               |
//|------------------------------------------------------------------+
#property copyright "renexxxx"
#property link      "http://www.stevehopwoodforex.com/"
#property version   "2.10"
#property strict

//--- .. Update: 2019.04.08
//--- ..         Apparently, ForexFactory have (temporarily?) moved the XML
//--- ..         to a new location. So, this code first checks feedURL1 and if empty
//--- ..         loads feedURL2. Yay! Thanks FF, for sending us on a treasure hunt!
//--- ..         Thanks Thomas for advising me! You are a STAR!
//--- .. Update: 2019.05.06
//--- ..         Rewritten most of the code now using the excellent EasyXML library

#define feedURL1  "http://www.forexfactory.com/ffcal_week_this.xml"
#define feedURL2  "https://nfs.faireconomy.media/ff_calendar_thisweek.xml"

#include <EasyXML\EasyXml.mqh>

enum NEWS_IMPACT {
   IMPACT_UNKNOWN    = -1,
   IMPACT_LOW        =  1,
   IMPACT_MEDIUM     =  2,
   IMPACT_HIGH       =  3
};

struct NewsItem {
   string        m_title;
   string        m_country;
   datetime      m_datetime;
   NEWS_IMPACT   m_impact;
   string        m_forecast;
   string        m_previous;
};

void getNews( NewsItem &news[] ) {

   // Clear the old items
   ArrayFree(news);
   ArrayResize(news, 0, 100);

   // Create object of class CEasyXml
   CEasyXml EasyXmlDocument;

   // Set debugging
   EasyXmlDocument.setDebugging(false);
   
   // Clear the document root
   EasyXmlDocument.Clear();

   // Walking through the dom tree inline

   bool success = EasyXmlDocument.loadXmlFromUrl(feedURL2);
   
   if ( success ) {
   
      CEasyXmlNode *rootNode=EasyXmlDocument.getDocumentRoot();

      if ( CheckPointer(rootNode) != POINTER_INVALID ) {
      
         CEasyXmlNode *childNode = rootNode.FirstChild();

         while ( CheckPointer(childNode) != POINTER_INVALID ) {

            if ( childNode.getName() == "event" ) {
   
               // Create a new NewsItem object
               int currentSize = ArraySize(news);
               ArrayResize(news, currentSize+1, 100);
   
               string date = "";
               string time = "";               
   
               CEasyXmlNode *subNode = childNode.FirstChild();
   
               while ( CheckPointer(subNode) != POINTER_INVALID ) {

                  // Fill the appropriate attribute of myNewsItem
                  if ( subNode.getName() == "title" ) {
                     news[currentSize].m_title = subNode.getValue();
                  }
                  else if ( subNode.getName() == "country" ) {
                     news[currentSize].m_country = subNode.getValue();
                  }
                  else if ( subNode.getName() == "date" ) {
                     date = subNode.getValue();
                  }
                  else if ( subNode.getName() == "time" ) {
                     time = subNode.getValue();
                  }
                  else if ( subNode.getName() == "impact" ) {
                     news[currentSize].m_impact = FFStrToImpact( subNode.getValue() );
                  }
                  else if ( subNode.getName() == "forecast" ) {
                     news[currentSize].m_forecast = subNode.getValue();
                  }
                  else if ( subNode.getName() == "previous" ) {
                     news[currentSize].m_previous = subNode.getValue();
                  }
                  
                  subNode = subNode.Next();
               }
               
               if ( date != "" && time != "" ) {
                  news[currentSize].m_datetime = FFStrToDate(date,time);
               }
            }
            childNode = childNode.Next();
         }
      }
   }
}

datetime FFStrToDate(string ffDate, string ffTime) {

   // ffDate is in the format: mm-dd-yyyy
   // ffTime is in the format: hh:mm[am|pm]
   
   static datetime gmtOffSet = TimeGMTOffset();

   MqlDateTime dateStruct;
   TimeToStruct( TimeCurrent(), dateStruct );
   
   string tempStr = ffDate;
   StringReplace(tempStr," ",""); // Remove the spaces

   if ( StringLen(tempStr) >= 10 ) {
      // Set the month
      dateStruct.mon  = StrToInteger( StringSubstr(tempStr, 0, 2) );
      
      // Set the day
      dateStruct.day  = StrToInteger( StringSubstr(tempStr, 3, 2) );
      
      // Set the year
      dateStruct.year = StrToInteger( StringSubstr(tempStr, 6, 4) );
   }

   tempStr = ffTime;
   StringReplace(tempStr," ",""); // Remove the spaces
   if ( StringFind(tempStr,":") == 1 ) tempStr = "0" + tempStr;    // Add a leading '0' for times like '9:05' -> '09:05'

   bool isPM = false;
   bool isAM = false;
   if ( StringLen(tempStr) > 5 ) {
   
      string ampm = StringSubstr(tempStr,5,2);
      StringToUpper(ampm);
      
      if ( ampm == "PM" ) isPM = true;
      else if ( ampm == "AM" ) isAM = true;
   }
   
   if ( StringLen(tempStr) >= 5 ) {
   
      // Set the hour
      dateStruct.hour = StrToInteger(StringSubstr(tempStr,0,2) );
      if ( isPM && ( (dateStruct.hour > 0) && (dateStruct.hour < 12) ) ) {
         dateStruct.hour += 12;
      }
      if ( isAM &&  (dateStruct.hour == 12) ) {
         dateStruct.hour -= 12;
      }
      
      // Set the minute
      dateStruct.min = StrToInteger(StringSubstr(tempStr,3,2) );
   }
   
   return( StructToTime(dateStruct) );
}

NEWS_IMPACT FFStrToImpact(string impact) {

   NEWS_IMPACT result = IMPACT_UNKNOWN;

   string tempStr = impact;
   StringReplace(tempStr," ",""); // Remove the spaces
   StringToUpper(tempStr);        // Upper case
   
   if ( tempStr == "LOW" ) result = IMPACT_LOW;
   else if ( tempStr == "MEDIUM" ) result = IMPACT_MEDIUM;
   else if ( tempStr == "HIGH" ) result = IMPACT_HIGH;
   
   return(result);
}